/* instruction-level simulator for RiSC Level-0 */

#include <stdio.h>
#include <string.h>

#define MAXLINELENGTH 1000
#define MEMSIZE 512	/* maximum number of memory words */

short Memory[ MEMSIZE ];
void printState();

main(int argc, char *argv[])
{
	FILE *filePtr;
	short address;
	char line[MAXLINELENGTH];

	if (argc != 2) { 
		printf("error: usage: %s <machine-code file>\n", argv[0]); 
		exit(1);
	}

	filePtr = fopen(argv[1], "r");
	if (filePtr == NULL) {
		printf("error: can't open file %s", argv[1]);
		perror("fopen");
		exit(1);
	}

	/* read in the entire machine-code file into memory */
	printf("Initializing memory ...\n");
	for (	address = 0;
			fgets(line, MAXLINELENGTH, filePtr) != NULL;
			address++) {

		if (sscanf(line, "%hx", &Memory[address]) != 1) {
			printf("error in reading address %d\n", address);
			exit(1);
		}
		printf("Memory[%hd] = %hx\n", address, Memory[address]);
	}

	printState();
}

int PC;
int rf[8];
int cycle;
const int count = 16;


/*
 * here is an example print-out function ... it assumes
 * that you have the following global variables:
 *   "cycle"  = cycle count
 *   "PC"     = program counter
 *   "rf"     = register file
 *   "Memory" = main memory
 */
void printState()
{
	int i;
	printf("State at the end of cycle %ld\n", cycle);
	printf("   PC = %hd\n", PC);
	printf("   Regs:\n");
	for (i=0; i<8; i++) {
		printf("    %02d %hd\n", i, rf[i]);
	}
	printf("   Memory:\n");
	for (i=0; i<count; i++) {
		printf("    %02d %04hx\n", i, Memory[i]);
	}
	printf("\n");
}

